/*  "Expressive" compiler
 *
 *  Common base class for the compiler programs
 *  Philippe Altherr
 *
 *  $Id: AbstractMain.java,v 1.2 2003/03/20 09:44:12 mihaylov Exp $
 */

package expressive;

import java.io.InputStream;
import java.io.OutputStream;
import java.io.PrintStream;
import java.io.PrintWriter;

/**
 * This class is designed as a common base class for all the
 * of the compiler. It analyzes the command line parameters,
 * creates the unique instance of class Global, opens and
 * closes the input and output files.
 */
public abstract class AbstractMain {

    //########################################################################
    // Instance variables

    /** Exception to be thrown to abort the compiler upon fatal error. */
    public final Error abort = new Error();

    /** The name of the source file or 'null' if not specified. */
    public String source = null;

    /** The name of the output file or 'null' if not specified. */
    public String object = null;

    /** Indicates the presence of the '-debug' option. */
    public boolean debug = false;

    //########################################################################
    // Methods

    /** Invokes the main program. */
    public abstract void run(Global global, InputStream in, PrintWriter out);

    /** Analyze the arguments and launch the main program. */
    public void run(String[] args) {
        Global global = null;
        try {
            handle(args);
            global = global();
            run(global);
        } catch (Error exception) {
            if (exception != abort) throw exception;
        }
        if (global != null) {
            int errors = global.errors();
            if (errors > 0) {
                System.err.println();
                System.err.print(errors + " error");
                if (errors > 1) System.err.print("s");
                System.err.println();
            }
        }
    }

    /** Open and close the various files. */
    public void run(Global global) {
        InputStream in = null;
        OutputStream out = null;
        PrintWriter writer = null;
        try {
            in = global.openInput(source);
            out = global.openOutput(object);
            writer = new PrintWriter(out, out == System.out);
            run(global, in, writer);
        } finally {
            if (writer != null) writer.close();
            if (out != null) global.close(out);
            if (in != null) global.close(in);
        }
    }

    /** Create the unique Global instance. */
    public Global global() {
        return new Global(this);
    }

    /** Print the error message and abort the execution. */
    public Error abort(String message) {
        System.err.println("error : " + message);
        printUsage(System.err);
        throw abort;
    }

    /** Handle command line parameters. */
    public void handle(String[] args) {
            for (int i = 0; i < args.length; ) {
                if (args[i].startsWith("-") && args[i].length() > 1) {
                    i += handleOption(args, i);
                } else {
                    handleFile(args[i]);
                    i += 1;
                }
            }
    }

    /** Handle command line options and return
     *  the number of the consumed arguments. */
    public int handleOption(String[] args, int i) {
        if (args[i].equals("-debug")) {
            debug = true;
            return 1;
        } else if (args[i].equals("-?") || args[i].equals("-help")) {
            printHelp(System.out);
            throw abort;
        } else {
            throw abort("unknown option " + args[i]);
        }
    }

    /** Handle the argument as a file name. */
    public void handleFile(String filename) {
        if (source == null) {
            source = filename;
        } else if (object == null) {
            object = filename;
        } else {
            throw abort("Don't know what to do with argument: " + filename);
        }
    }

    /** Print usage info. */
    public void printUsage(PrintStream out) {
        out.println("usage: java " + getClass().getName() +
            " <options> [source-file [output-file]]");
    }

    /** Print help info. */
    public void printHelp(PrintStream out) {
        printUsage(out);
        out.println("with the following options :");
        out.println("  -debug    Print the debug messages");
        out.println("  -? -help  Show this help");
    }

    //########################################################################
}
